/*
 * Copyright (C) 2024 sillysagiri
 *
 * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
#pragma once

#include "stb_image.h"
#include <cstdint>
#include <filesystem>
#include <string>
#include <vector>

// wporkaround surpress clangd pragma pack warning
// https://github.com/clangd/clangd/issues/1167
static_assert(true);

#pragma pack(push, 1)
struct Metadata {
	uint64_t header = 0x676D69796C6C6973;
	uint8_t version = 13;
	uint8_t format;
	uint16_t width;
	uint16_t height;
	uint16_t palette_count;
	uint32_t palette_hash;
	uint32_t original_size;
	uint32_t compress_size;
};
#pragma pack(pop)

enum Format
{
	Format_RGB_256 = 0,
	Format_RGB_16,
	Format_INDEXED_4,
	Format_INDEXED_16,
	Format_INDEXED_256,
	Format_INDEXED_32A3,
	Format_INDEXED_8A5,
	Format_PALETTE_16,
};

struct Image {
	int width, height, comp;
	stbi_uc *data;

	constexpr Image() : width(0), height(0), comp(0), data(nullptr) {}
	Image(const std::filesystem::path &path);
	~Image();

	void Load(const std::filesystem::path &path);
};

struct IndexedImage
{
	int width, height;
	uint8_t *data;

	constexpr IndexedImage() : width(0), height(0), data(nullptr) {}
	IndexedImage(int width, int height);
	~IndexedImage();

	void Load(int width, int height);
};

struct Color {
	uint8_t a;
	uint8_t r;
	uint8_t g;
	uint8_t b;
};

struct Options {
	std::vector<std::string> path_input;
	std::string path_output;
	std::string format;
};

void Verify();
void Convert();

uint16_t RGB16(uint8_t a, uint8_t r, uint8_t g, uint8_t b);
void QuantizeImage(const int numColor, std::vector<Image> &images, std::vector<Color> &palette, std::vector<IndexedImage> &indexes);
void BitPacking(const uint8_t bpp, IndexedImage &img, uint32_t &length);